@@ -7,6 +7,7 @@ import tables
 import warnings
 
 import numpy as np
+import scipy.interpolate
 from scipy.signal import get_window
 
 import matplotlib.pyplot as plt
@@ -585,7 +586,7 @@ class CartImageCube(ImageCube):
                 fmhz = self.freqs[0] + (self.freqs[-1] - self.freqs[0]) / 2.
             elif fmhz == 'first':
                 fmhz = self.freqs[0]
-            elif fmhz == 'first':
+            elif fmhz == 'last':
                 fmhz = self.freqs[-1]
 
             i = np.argmin(abs(self.freqs - fmhz * 1e6))
@@ -1214,7 +1215,7 @@ class CartDataCube(DataCube):
                 group = h5_file.create_group("/", 'weights', 'Weights cube (n_freqs, n_vis)')
                 self.weights.save_to_hd5(h5_file, group)
 
-    def new_with_data(self, data, cov_err=None, weights=None, freqs=None):
+    def new_with_data(self, data, cov_err=None, weights=None, freqs=None, interpolate_weights=True):
         """Create a new CartDataCube using data and cov_err"""
         if freqs is None:
             freqs = self.freqs
@@ -1228,6 +1229,11 @@ class CartDataCube(DataCube):
         if weights is not None:
             weights = weights.copy()
 
+            if weights.data.shape[0] != len(freqs):
+                w = scipy.interpolate.interp1d(weights.freqs, weights.data, bounds_error=False, 
+                                               fill_value='extrapolate', axis=0)(freqs)
+                weights = weights.new_with_data(w, freqs=freqs)
+
         return CartDataCube(data, self.uu, self.vv, freqs, self.meta.copy(),
                             cov_err=cov_err, weights=weights)
 
@@ -1480,11 +1486,11 @@ class CartDataCube(DataCube):
 
         if action_fct is None:
             if fmhz == 'med':
-                fmhz = self.freqs[0] + (self.freqs[-1] - self.freqs[0]) / 2.
+                fmhz = (self.freqs[0] + (self.freqs[-1] - self.freqs[0]) / 2.) * 1e-6
             elif fmhz == 'first':
-                fmhz = self.freqs[0]
+                fmhz = self.freqs[0] * 1e-6
             elif fmhz == 'last':
-                fmhz = self.freqs[-1]
+                fmhz = self.freqs[-1] * 1e-6
 
             i = np.argmin(abs(self.freqs - fmhz * 1e6))
             d = d[i]
@@ -1679,9 +1685,6 @@ class NoiseStdCube(CartDataCube):
 class CartWeightCube(CartDataCube):
 
     def __init__(self, weight_cube, uu, vv, freqs, meta, uv_scale=None, freqs_n_nights=None):
-        assert 'PEINTTIM' in meta
-        assert 'PETOTTIM' in meta
-
         CartDataCube.__init__(self, weight_cube, uu, vv, freqs, meta)
         if uv_scale is None:
             self.unscale()
