@@ -145,16 +145,16 @@ class TelescopeSimu(object):
 
         return np.array(uu), np.array(vv), np.array(ww)
 
-    def simu_ungridded_uv(self, uu_meter, vv_meter):
-        from ps_eor import psutil, datacube
+    # def simu_ungridded_uv(self, uu_meter, vv_meter):
+    #     from ps_eor import psutil, datacube
 
-        meta = datacube.ImageMetaData.from_res(res, shape)
-        meta.wcs.wcs.cdelt[2] = psutil.robust_freq_width(self.freqs)
-        meta.set('PEINTTIM', self.timeres)
-        meta.set('PETOTTIM', (self.har - self.hal) * 3600)
-        w_cube = datacube.CartWeightCube(weights, g_uu, g_vv, self.freqs, meta)
+    #     meta = datacube.ImageMetaData.from_res(res, shape)
+    #     meta.wcs.wcs.cdelt[2] = psutil.robust_freq_width(self.freqs)
+    #     meta.set('PEINTTIM', self.timeres)
+    #     meta.set('PETOTTIM', (self.har - self.hal) * 3600)
+    #     w_cube = datacube.CartWeightCube(weights, g_uu, g_vv, self.freqs, meta)
 
-        return TelescopGriddedUV(w_cube, self)
+    #     return TelescopGriddedUV(w_cube, self)
 
 
     def simu_gridded_uv(self, uu_meter, vv_meter, fov, oversampling_factor=4, min_weight=10):
@@ -363,50 +363,50 @@ class NenuFAR(TelescopeSimu):
         return 2 * const.k_B.value / a_eff * 1e26 * tsys
 
 
-class LWA(TelescopeSimu):
+# class LWA(TelescopeSimu):
 
-    name = 'nenufar'
-    umin = 6
-    umax = 60
-    fov = 16
-    du = 4
-    pb_name = name
+#     name = 'nenufar'
+#     umin = 6
+#     umax = 60
+#     fov = 16
+#     du = 4
+#     pb_name = name
 
-    def get_stat_pos_file(self):
-        return os.path.join(INSTRU_DIR, 'nenufar52_statpos.data')
+#     def get_stat_pos_file(self):
+#         return os.path.join(INSTRU_DIR, 'nenufar52_statpos.data')
 
-    def sky_temperature(self, freq, tsys_sky=60):
-        lamb = const.c.value / freq
-        return tsys_sky * lamb ** 2.55
+#     def sky_temperature(self, freq, tsys_sky=60):
+#         lamb = const.c.value / freq
+#         return tsys_sky * lamb ** 2.55
 
-    def inst_temperature(self, freq):
-        """ Instrument temperature at a given frequency ``freq``.
+#     def inst_temperature(self, freq):
+#         """ Instrument temperature at a given frequency ``freq``.
 
-            From: https://github.com/AlanLoh/nenupy/blob/master/nenupy/instru/instru.py
-        """
-        lna_sky = np.array([
-            5.0965, 2.3284, 1.0268, 0.4399, 0.2113, 0.1190, 0.0822, 0.0686,
-            0.0656, 0.0683, 0.0728, 0.0770, 0.0795, 0.0799, 0.0783, 0.0751,
-            0.0710, 0.0667, 0.0629, 0.0610, 0.0614, 0.0630, 0.0651, 0.0672,
-            0.0694, 0.0714, 0.0728, 0.0739, 0.0751, 0.0769, 0.0797, 0.0837,
-            0.0889, 0.0952, 0.1027, 0.1114, 0.1212, 0.1318, 0.1434, 0.1562,
-            0.1700, 0.1841, 0.1971, 0.2072, 0.2135, 0.2168, 0.2175, 0.2159,
-            0.2121, 0.2070, 0.2022, 0.1985, 0.1974, 0.2001, 0.2063, 0.2148,
-            0.2246, 0.2348, 0.2462, 0.2600, 0.2783, 0.3040, 0.3390, 0.3846,
-            0.4425, 0.5167, 0.6183, 0.7689, 1.0086, 1.4042, 2.0732
-        ])
-        lna_freqs = (np.arange(71) + 15) * 1e6
-        return self.sky_temperature(freq) * scipy.interpolate.interp1d(lna_freqs, lna_sky,
-                                                                       bounds_error=False,
-                                                                       fill_value='extrapolate')(freq)
+#             From: https://github.com/AlanLoh/nenupy/blob/master/nenupy/instru/instru.py
+#         """
+#         lna_sky = np.array([
+#             5.0965, 2.3284, 1.0268, 0.4399, 0.2113, 0.1190, 0.0822, 0.0686,
+#             0.0656, 0.0683, 0.0728, 0.0770, 0.0795, 0.0799, 0.0783, 0.0751,
+#             0.0710, 0.0667, 0.0629, 0.0610, 0.0614, 0.0630, 0.0651, 0.0672,
+#             0.0694, 0.0714, 0.0728, 0.0739, 0.0751, 0.0769, 0.0797, 0.0837,
+#             0.0889, 0.0952, 0.1027, 0.1114, 0.1212, 0.1318, 0.1434, 0.1562,
+#             0.1700, 0.1841, 0.1971, 0.2072, 0.2135, 0.2168, 0.2175, 0.2159,
+#             0.2121, 0.2070, 0.2022, 0.1985, 0.1974, 0.2001, 0.2063, 0.2148,
+#             0.2246, 0.2348, 0.2462, 0.2600, 0.2783, 0.3040, 0.3390, 0.3846,
+#             0.4425, 0.5167, 0.6183, 0.7689, 1.0086, 1.4042, 2.0732
+#         ])
+#         lna_freqs = (np.arange(71) + 15) * 1e6
+#         return self.sky_temperature(freq) * scipy.interpolate.interp1d(lna_freqs, lna_sky,
+#                                                                        bounds_error=False,
+#                                                                        fill_value='extrapolate')(freq)
 
-    def get_sefd(self, freq, tsys_sky=60):
-        d = 5.5
-        lamb = const.c.value / freq
-        tsys = self.sky_temperature(freq, tsys_sky) + self.inst_temperature(freq)
-        a_eff = 19 * np.min([lamb ** 2 / 3., np.ones_like(lamb) * np.pi * d ** 2 / 4.], axis=0)
+#     def get_sefd(self, freq, tsys_sky=60):
+#         d = 5.5
+#         lamb = const.c.value / freq
+#         tsys = self.sky_temperature(freq, tsys_sky) + self.inst_temperature(freq)
+#         a_eff = 19 * np.min([lamb ** 2 / 3., np.ones_like(lamb) * np.pi * d ** 2 / 4.], axis=0)
 
-        return 2 * const.k_B.value / a_eff * 1e26 * tsys
+#         return 2 * const.k_B.value / a_eff * 1e26 * tsys
 
 
 
@@ -419,3 +419,13 @@ class NenuFARFull(NenuFAR):
     def get_stat_pos_file(self):
         return os.path.join(INSTRU_DIR, 'nenufar_full_statpos.data')
 
+
+# class NenuFAR80(NenuFAR):
+
+#     name = 'nenufar_80'
+#     pb_name = 'nenufar'
+
+
+#     def get_stat_pos_file(self):
+#         return os.path.join(INSTRU_DIR, 'nenufar80_statpos.data')
+
