@@ -2,6 +2,7 @@
 #
 # Authors: F.Mertens
 
+import os
 import operator
 import numpy as np
 
@@ -114,7 +115,7 @@ class EorBinList(object):
 
         assert name in self.windows, "Error No EoR bin with name '%s'" % name
         assert freqs is not None, "The frequencies need to be supplied either at the " \
-            "initialization of the object or at the method level."
+                                  "initialization of the object or at the method level."
 
         fmhz = freqs * 1e-6
         fmhz_start, fmhz_end, fmhz_fg_start, fmhz_fg_end = self.windows[name]
@@ -122,6 +123,7 @@ class EorBinList(object):
         slice_bin_fg = psutil.get_freq_slice(fmhz, fmhz_fg_start, fmhz_fg_end)
 
         if not len(freqs[slice_bin]) > 1:
+            print(f'Warning: datacube frequency range ({fmhz.min():.1f}-{fmhz.max():.1f} MHz) does not match EoR window {fmhz_start:.1f}-{fmhz_end:.1f}) MHz')
             return None
 
         return EorBin(name, freqs[slice_bin], freqs[slice_bin_fg])
@@ -295,30 +297,32 @@ class PowerSpectraBuilder(object):
         elif not isinstance(ps_config, PowerSpectraConfig):
             ps_config = PowerSpectraConfig.load(ps_config)
 
-        if eor_bin_list is None:
-            eor_bin_list = EorBinList()
-            eor_bin_list.add_freq('0', 10, 200)
-        elif not isinstance(eor_bin_list, EorBinList):
+        if not isinstance(eor_bin_list, EorBinList):
             eor_bin_list = EorBinList.load(eor_bin_list)
 
         self.ps_config = ps_config
         self.eor_bin_list = eor_bin_list
 
     def get(self, cube, eor_bin_name=None, z=None, fmhz_range=None, **kargs):
-        if z is not None:
-            mfreq = psutil.z_to_freq(z) * 1e-6
-            fmin = mfreq - 4
-            fmax = mfreq + 4
-            z_eor_bin_list = EorBinList()
-            z_eor_bin_list.add_freq(0, fmin, fmax, fmin, fmax)
-            eor = z_eor_bin_list.get(0, freqs=cube.freqs)
-        elif fmhz_range is not None:
-            fmin, fmax = fmhz_range
-            f_eor_bin_list = EorBinList()
-            f_eor_bin_list.add_freq(0, fmin, fmax, fmin, fmax)
-            eor = f_eor_bin_list.get(0, freqs=cube.freqs)
-        else:
+        assert (eor_bin_name is not None) + (z is not None) + (fmhz_range is not None) <= 1, 'Only one of eor_bin_name, z or fmhz_range may be given.'
+        
+        if eor_bin_name is not None and self.eor_bin_list is not None:
             eor = self.eor_bin_list.get(eor_bin_name, freqs=cube.freqs)
+        else:
+            fmhz = cube.freqs * 1e-6
+            if z is not None:
+                mfreq = psutil.z_to_freq(z) * 1e-6
+                fmin = mfreq - 5
+                fmax = mfreq + 5
+            elif fmhz_range is not None:
+                fmin, fmax = fmhz_range
+            else:
+                fmin = fmhz.min()
+                fmax = fmhz.max()
+            slice_bin = psutil.get_freq_slice(fmhz, fmin, fmax)
+            if len(fmhz[slice_bin]) <= 3:
+                print(f'Warning: The datacube has only {len(fmhz[slice_bin])} frequency channel for the chosen frequency bin.')
+            eor = EorBin(0, cube.freqs[slice_bin], cube.freqs[slice_bin])
 
         ps_config = self.ps_config.copy()
         ps_config.parse_dict(kargs)
@@ -329,6 +333,7 @@ class PowerSpectraBuilder(object):
             ps_gen = PowerSpectraCart(eor, ps_config, pb)
         elif isinstance(cube, sphcube.SphDataCube):
             # TODO: get PB from config
+            print('Warning: no primary beam is set in ps_gen for this mode. You need to set it manually.')
             pb = sphcube.NoPrimaryBeam()
 
             ps_gen = PowerSpectraSph(eor, ps_config, pb)
@@ -485,13 +490,25 @@ class BasePowerSpectra(object):
         '''Primary Beam normalization factor'''
         return NotImplementedError()
 
-    def get_ps2d(self, data_cube):
+    def get_ps2d(self, data_cube, **kargs):
         return NotImplementedError()
 
-    def get_ps(self, data_cube):
+    def get_ps(self, data_cube, **kargs):
         return NotImplementedError()
 
-    def get_ps3d(self, kbins, data_cube):
+    def get_variance(self, data_cube, **kargs):
+        return NotImplementedError()
+
+    def get_ps3d(self, kbins, data_cube, **kargs):
+        return NotImplementedError()
+
+    def get_cross_ps2d(self, kbins, data_cube1, data_cube2, **kargs):
+        return NotImplementedError()
+
+    def get_cross_ps(self, kbins, data_cube1, data_cube2, **kargs):
+        return NotImplementedError()
+
+    def get_cross_variance(self, kbins, data_cube1, data_cube2, **kargs):
         return NotImplementedError()
 
     def get_ps2d_norm(self, data_cube):
@@ -503,8 +520,8 @@ class BasePowerSpectra(object):
                            weighted='default'):
         ''' Return 2D coherence of ft_cube1 and ft_cube2 (n_freqs, n_vis) '''
         cross = self.get_cross_ps2d(ft_cube1, ft_cube2, with_cov_err, weighted=weighted)
-        ps2d_1 = self.get_ps2d(ft_cube1, with_cov_err, weighted=weighted)
-        ps2d_2 = self.get_ps2d(ft_cube2, with_cov_err, weighted=weighted)
+        ps2d_1 = self.get_ps2d(ft_cube1, with_cov_err=with_cov_err, weighted=weighted)
+        ps2d_2 = self.get_ps2d(ft_cube2, with_cov_err=with_cov_err, weighted=weighted)
 
         if cross_square:
             cross_coh = cross.data ** 2 / (ps2d_1.data * ps2d_2.data)
@@ -517,8 +534,8 @@ class BasePowerSpectra(object):
     def get_coherence_ps(self, ft_cube1, ft_cube2, with_cov_err=False, weighted='default'):
         ''' Return 2D coherence of ft_cube1 and ft_cube2 (n_freqs, n_vis) '''
         cross = self.get_cross_ps(ft_cube1, ft_cube2, with_cov_err, weighted=weighted)
-        ps2d_1 = self.get_ps(ft_cube1, with_cov_err, weighted=weighted)
-        ps2d_2 = self.get_ps(ft_cube2, with_cov_err, weighted=weighted)
+        ps2d_1 = self.get_ps(ft_cube1, with_cov_err=with_cov_err, weighted=weighted)
+        ps2d_2 = self.get_ps(ft_cube2, with_cov_err=with_cov_err, weighted=weighted)
 
         cross_coh = cross.data ** 2 / (ps2d_1.data * ps2d_2.data)
 
@@ -528,8 +545,8 @@ class BasePowerSpectra(object):
     def get_coherence_variance(self, ft_cube1, ft_cube2, with_cov_err=False, weighted='default'):
         ''' Return 2D coherence of ft_cube1 and ft_cube2 (n_freqs, n_vis) '''
         cross = self.get_cross_variance(ft_cube1, ft_cube2, with_cov_err, weighted=weighted)
-        var_1 = self.get_variance(ft_cube1, with_cov_err, weighted=weighted)
-        var_2 = self.get_variance(ft_cube2, with_cov_err, weighted=weighted)
+        var_1 = self.get_variance(ft_cube1, with_cov_err=with_cov_err, weighted=weighted)
+        var_2 = self.get_variance(ft_cube2, with_cov_err=with_cov_err, weighted=weighted)
 
         return Variance(cross.data ** 2 / (var_1.data * var_2.data), np.zeros_like(cross.data),
                         cross.freqs)
@@ -538,8 +555,8 @@ class BasePowerSpectra(object):
                            cross_square=True, weighted='default'):
         ''' Return 2D coherence of ft_cube1 and ft_cube2 (n_freqs, n_vis) '''
         cross = self.get_cross_ps3d(kbins, ft_cube1, ft_cube2, with_cov_err, weighted=weighted)
-        ps3d_1 = self.get_ps3d(kbins, ft_cube1, with_cov_err, weighted=weighted)
-        ps3d_2 = self.get_ps3d(kbins, ft_cube2, with_cov_err, weighted=weighted)
+        ps3d_1 = self.get_ps3d(kbins, ft_cube1, with_cov_err=with_cov_err, weighted=weighted)
+        ps3d_2 = self.get_ps3d(kbins, ft_cube2, with_cov_err=with_cov_err, weighted=weighted)
 
         if cross_square:
             cross_coh = cross.data ** 2 / (ps3d_1.data * ps3d_2.data)
@@ -730,8 +747,16 @@ class PowerSpectraCart(BasePowerSpectra):
                                                                                  half=self.ps2d_pos_only,
                                                                                  weight_cube=weight_cube)
 
+        _, _, ps2d_1_err, _, _ = pscart.get_2d_cross_power_spectra(f_delay, dft_cube1, dft_cube1, data_cube1.uu,
+                                                                   data_cube1.vv, self.el, half=self.ps2d_pos_only,
+                                                                   weight_cube=weight_cube)
+
+        _, _, ps2d_2_err, _, _ = pscart.get_2d_cross_power_spectra(f_delay, dft_cube1, dft_cube1, data_cube1.uu,
+                                                                   data_cube1.vv, self.el, half=self.ps2d_pos_only,
+                                                                   weight_cube=weight_cube)
+
         ps2d = ps2d * ps2d_norm
-        ps2d_err = ps2d_err * ps2d_norm
+        ps2d_err = np.sqrt(ps2d_1_err * ps2d_2_err) * ps2d_norm
         ps2d, ps2d_err = self._handle_cov_err_cross(ps2d, ps2d_err, self.ps2d_cov_cache,
                                                     data_cube1, data_cube2, with_cov_err,
                                                     weighted=weighted)
@@ -777,13 +802,19 @@ class PowerSpectraCart(BasePowerSpectra):
         ps, ps_err, n_eff, ps_w = pscart.get_cross_power_spectra(data_cube1.data, data_cube2.data, data_cube1.uu,
                                                                  data_cube1.vv, self.el, weight_cube=weight_cube)
 
+        _, ps_1_err, _, _ = pscart.get_cross_power_spectra(data_cube1.data, data_cube1.data, data_cube1.uu,
+                                                           data_cube1.vv, self.el, weight_cube=weight_cube)
+
+        _, ps_2_err, _, _ = pscart.get_cross_power_spectra(data_cube2.data, data_cube2.data, data_cube1.uu,
+                                                           data_cube1.vv, self.el, weight_cube=weight_cube)
+
         ps = ps * ps_norm
-        ps_err = ps_err * ps_norm
+        ps_err = np.sqrt(ps_1_err * ps_2_err) * ps_norm
 
         ps, ps_err = self._handle_cov_err_cross(ps, ps_err, self.ps_cov_cache, data_cube1,
                                                 data_cube2, with_cov_err, weighted=weighted)
 
-        return SpatialPowerSpectra(ps, ps_err, self.eor.freqs, self.el, self.k_per, n_eff=n_eff, ps_w=ps_w)
+        return SpatialPowerSpectra(ps, ps_err, data_cube1.freqs, self.el, self.k_per, n_eff=n_eff, ps_w=ps_w)
 
     def get_variance(self, data_cube, with_cov_err=True, weighted='default'):
         cl = self.get_cl(data_cube, with_cov_err=with_cov_err, weighted=weighted)
@@ -807,7 +838,7 @@ class PowerSpectraCart(BasePowerSpectra):
 
         ps2d_norm = self.get_ps2d_norm(data_cube)
         ps_err_norm = self.get_ps_err_norm(data_cube)
-        f_delay, dft_cube = self.delay_transform(data_cube)
+        _, dft_cube = self.delay_transform(data_cube)
 
         weight_cube = self.get_weights(data_cube, weighted, delay_transform=True,
                                        cov_err=data_cube.cov_err)
@@ -841,8 +872,8 @@ class PowerSpectraCart(BasePowerSpectra):
         data_cube2 = self.eor.get_slice(data_cube2)
 
         ps2d_norm = self.get_ps2d_norm(data_cube1)
-        f_delay, dft_cube2 = self.delay_transform(data_cube1)
-        f_delay, dft_cube1 = self.delay_transform(data_cube2)
+        _, dft_cube2 = self.delay_transform(data_cube1)
+        _, dft_cube1 = self.delay_transform(data_cube2)
 
         weight_cube = self.get_weights(data_cube1, weighted, delay_transform=True)
 
@@ -853,15 +884,21 @@ class PowerSpectraCart(BasePowerSpectra):
 
         k_mean, k_std, dsp, dsp_err, n_eff = pscart.get_3d_cross_power_spectre(dft_cube1, dft_cube2, kbins,
                                                                                k_per, k_par, weight_cube=weight_cube)
+        _, _, _, dsp1_err, _ = pscart.get_3d_cross_power_spectre(
+            dft_cube1, dft_cube1, kbins, k_per, k_par, weight_cube=weight_cube,
+            w_square=self.config.psf_weights_square)
+        _, _, _, dsp2_err, _ = pscart.get_3d_cross_power_spectre(
+            dft_cube2, dft_cube2, kbins, k_per, k_par, weight_cube=weight_cube,
+            w_square=self.config.psf_weights_square)
 
         dsp = dsp * ps2d_norm
-        dsp_err = dsp_err * ps2d_norm
+        dsp_err = np.sqrt(dsp1_err * dsp2_err) * ps2d_norm
 
         dsp, dsp_err = self._handle_cov_err_cross(dsp, dsp_err, self.ps3d_cov_cache,
                                                   data_cube1, data_cube2, with_cov_err,
                                                   kbins, weighted=weighted)
 
-        return SphericalPowerSpectra(dsp, dsp_err, kbins, k_mean, n_eff=n_eff)
+        return SphericalPowerSpectra(dsp, dsp_err, kbins, k_mean, n_eff=n_eff, k_std=k_std)
 
 
 class PowerSpectraSph(BasePowerSpectra):
@@ -1685,36 +1722,6 @@ class MultiNight2DPowerSpectra(PowerSpectraMath):
         ax.set_ylabel(self.ylabel)
 
 
-class PsStacker(object):
-
-    def __init__(self, ps_gen, kbins):
-        self.all_ps3d = []
-        self.all_ps = []
-        self.all_ps2d = []
-        self.all_var = []
-        self.ps_gen = ps_gen
-        self.kbins = kbins
-
-    def add(self, cube):
-        self.all_ps3d.append(self.ps_gen.get_ps3d(self.kbins, cube))
-        self.all_ps.append(self.ps_gen.get_ps(cube))
-        self.all_ps2d.append(self.ps_gen.get_ps2d(cube))
-        self.all_var.append(self.ps_gen.get_variance(cube))
-
-    def get_ps(self):
-        return SpatialPowerSpectraMC(self.all_ps)
-
-    def get_ps2d(self):
-        return CylindricalPowerSpectraMC(self.all_ps2d)
-
-    def get_variance(self):
-        return VarianceMC(self.all_var)
-
-    def get_ps3d(self):
-        return SphericalPowerSpectraMC(self.all_ps3d)
-
-
-
 class PowerSpectraMC(object):
 
     def __init__(self, all_ps):
@@ -1824,6 +1831,16 @@ class SphericalPowerSpectraMC(PowerSpectraMC, SphericalPowerSpectra):
             for d, l in zip(a, ['k_bins', 'k_mean', 'k_std', 'data', 'err', 'n_eff']):
                 if d is not None:
                     hf.create_dataset(l, data=d)
+        self.save_to_txt(os.path.splitext(filename)[0] + '.txt')
+
+    def save_to_txt(self, filename):
+        array_data = np.array([self.k_bins[:-1], self.k_bins[1:], self.k_mean,
+                               self.k_std, self.med * 1e6, self.q2_5 * 1e6, self.q16 * 1e6, 
+                               self.q84 * 1e6, self.q97_5 * 1e6, self.n_eff]).T
+        header = 'Spherically averaged Power Spectra n_k=%s\n' % (len(self.k_mean))
+        header += (r'k_min (h cMpc^-1), k_max (h cMpc^-1), k_mean (h cMpc^-1), k_std (h cMpc^-1), '
+                   r'\Delta^2 (mK^2), \Delta_q2.5^2 (mK^2), \Delta_q16^2 (mK^2), \Delta_q84^2 (mK^2), \Delta_q97.5^2 (mK^2), N_eff\n')
+        np.savetxt(filename, array_data, fmt='%14.8f', header=header, delimiter=' ')
 
     @staticmethod
     def load(filename):
@@ -2065,3 +2082,49 @@ class SpatialPowerSpectraMC(PowerSpectraMC, SpatialPowerSpectra):
         all_ps = [SpatialPowerSpectra(d, e, freqs, el, k_per) for d, e in zip(data, err)]
 
         return VarianceMC(all_ps)
+
+
+class PsStacker(object):
+
+    def __init__(self, ps_gen, kbins):
+        self.all_ps3d = []
+        self.all_ps = []
+        self.all_ps2d = []
+        self.all_var = []
+        self.ps_gen = ps_gen
+        self.kbins = kbins
+
+    def add(self, cube):
+        if self.ps_gen is None or self.kbins is None:
+            print('Warning: can not add a datacube to a loaded PsStacker')
+            return
+
+        self.all_ps3d.append(self.ps_gen.get_ps3d(self.kbins, cube))
+        self.all_ps.append(self.ps_gen.get_ps(cube))
+        self.all_ps2d.append(self.ps_gen.get_ps2d(cube))
+        self.all_var.append(self.ps_gen.get_variance(cube))
+
+    def get_ps(self):
+        return SpatialPowerSpectraMC(self.all_ps)
+
+    def get_ps2d(self):
+        return CylindricalPowerSpectraMC(self.all_ps2d)
+
+    def get_variance(self):
+        return VarianceMC(self.all_var)
+
+    def get_ps3d(self):
+        return SphericalPowerSpectraMC(self.all_ps3d)
+
+    def save(self, dir_path, name):
+        self.get_ps3d().save(os.path.join(dir_path, f'{name}.ps3d.h5'))
+        self.get_ps2d().save(os.path.join(dir_path, f'{name}.ps2d.h5'))
+        self.get_variance().save(os.path.join(dir_path, f'{name}.variance.h5'))
+        self.get_ps().save(os.path.join(dir_path, f'{name}.ps.h5'))
+
+    def load(self, dir_path, name):
+        ps_stacker = PsStacker(None, None)
+        ps_stacker.all_ps3d = SphericalPowerSpectraMC.load(os.path.join(dir_path, f'{name}.ps3d.h5')).all_ps
+        ps_stacker.all_ps2d = CylindricalPowerSpectraMC.load(os.path.join(dir_path, f'{name}.ps2d.h5')).all_ps
+        ps_stacker.all_ps = SpatialPowerSpectraMC.load(os.path.join(dir_path, f'{name}.ps.h5')).all_ps
+        ps_stacker.all_var = VarianceMC.load(os.path.join(dir_path, f'{name}.variance.h5')).all_ps
